@@ -2,13 +2,12 @@ import datetime
 import logging
 import time
 from decimal import Decimal
+from typing import Union
 
 import ccxt
-
-# from credentials import CcxtConfig
 import pandas as pd
 
-from lumibot.entities import Bars
+from lumibot.entities import Asset, Bars
 
 from .data_source import DataSource
 
@@ -28,7 +27,8 @@ class CcxtData(DataSource):
     def _format_datetime(dt):
         return pd.Timestamp(dt).isoformat()
 
-    def __init__(self, api_keys, max_workers=20, chunk_size=100, **kwargs):
+    def __init__(self, config, max_workers=20, chunk_size=100, **kwargs):
+        super().__init__(**kwargs)
         self.name = "ccxt"
         self.max_workers = min(max_workers, 200)
 
@@ -38,31 +38,24 @@ class CcxtData(DataSource):
         self.chunk_size = min(chunk_size, 100)
 
         try:
-            exchange_class = getattr(ccxt, api_keys["exchange_id"])
+            exchange_class = getattr(ccxt, config["exchange_id"])
         except:
             raise Exception(
                 "Could not find exchange named '{}'. Are you sure you are spelling the exchange_id correctly?".format(
-                    api_keys["exchange_id"]
+                    config["exchange_id"]
                 )
             )
 
-        self.api_keys = api_keys
-        self.api = exchange_class(api_keys)
-        is_sandbox = True if "sandbox" not in api_keys else api_keys["sandbox"]
+        self.config = config
+        self.api = exchange_class(config)
+        is_sandbox = True if "sandbox" not in config else config["sandbox"]
         self.api.set_sandbox_mode(is_sandbox)
         self.api.load_markets()
         # Recommended two or less api calls per second.
         self.api.enableRateLimit = True
 
     def _pull_source_symbol_bars(
-        self,
-        asset,
-        length,
-        timestep=MIN_TIMESTEP,
-        timeshift=None,
-        quote=None,
-        exchange=None,
-        include_after_hours=True
+        self, asset, length, timestep=MIN_TIMESTEP, timeshift=None, quote=None, exchange=None, include_after_hours=True
     ):
         if exchange is not None:
             logging.warning(
@@ -70,13 +63,11 @@ class CcxtData(DataSource):
             )
 
         """pull broker bars for a given asset"""
-        response = self._pull_source_bars(
-            [asset], length, timestep=timestep, timeshift=timeshift, quote=quote
-        )
+        response = self._pull_source_bars([asset], length, timestep=timestep, timeshift=timeshift, quote=quote)
         return response[asset]
 
     def _pull_source_bars(
-        self, assets, length, timestep=MIN_TIMESTEP, timeshift=None, quote=None,  include_after_hours=True
+        self, assets, length, timestep=MIN_TIMESTEP, timeshift=None, quote=None, include_after_hours=True
     ):
         """pull broker bars for a list assets"""
         parsed_timestep = self._parse_source_timestep(timestep, reverse=True)
@@ -98,6 +89,39 @@ class CcxtData(DataSource):
 
         return result
 
+    def get_chains(self, asset: Asset, quote: Asset = None, exchange: str = None):
+        raise NotImplementedError(
+            "Lumibot CcxtData does not support historical options data. If you need this "
+            "feature, please use a different data source."
+        )
+
+    def get_historical_prices(
+        self, asset, length, timestep="", timeshift=None, quote=None, exchange=None, include_after_hours=True
+    ):
+        """Get bars for a given asset"""
+        if isinstance(asset, str):
+            asset = Asset(symbol=asset)
+
+        if not timestep:
+            timestep = self.get_timestep()
+
+        response = self._pull_source_symbol_bars(
+            asset,
+            length,
+            timestep=timestep,
+            timeshift=timeshift,
+            quote=quote,
+            exchange=exchange,
+            include_after_hours=include_after_hours,
+        )
+        if isinstance(response, float):
+            return response
+        elif response is None:
+            return None
+
+        bars = self._parse_source_symbol_bars(response, asset, quote=quote, length=length)
+        return bars
+
     def get_barset_from_api(self, api, symbol, freq, limit=None, end=None):
         """
         gets historical bar data for the given stock symbol
@@ -112,8 +136,7 @@ class CcxtData(DataSource):
         market = self.api.markets.get(symbol, None)
         if market is None:
             logging.error(
-                f"A request for market data for {symbol} was submitted. "
-                f"The market for that pair does not exist"
+                f"A request for market data for {symbol} was submitted. " f"The market for that pair does not exist"
             )
             return None
 
@@ -139,13 +162,9 @@ class CcxtData(DataSource):
 
         while True:
             cnt += 1
-            candles = self.api.fetch_ohlcv(
-                symbol, freq, since=curr_start, limit=loop_limit, params={}
-            )
+            candles = self.api.fetch_ohlcv(symbol, freq, since=curr_start, limit=loop_limit, params={})
 
-            df = pd.DataFrame(
-                candles, columns=["datetime", "open", "high", "low", "close", "volume"]
-            )
+            df = pd.DataFrame(candles, columns=["datetime", "open", "high", "low", "close", "volume"])
             df["datetime"] = pd.to_datetime(df["datetime"], unit="ms")
             df = df.set_index("datetime")
 
@@ -157,9 +176,7 @@ class CcxtData(DataSource):
             df_ret = df_ret.sort_index()
 
             if len(df) > 0:
-                last_curr_end = self.api.parse8601(
-                    df.index[-1].strftime("%Y-%m-%d %H:%M:%S")
-                )
+                last_curr_end = self.api.parse8601(df.index[-1].strftime("%Y-%m-%d %H:%M:%S"))
             else:
                 last_curr_end = None
 
@@ -191,11 +208,10 @@ class CcxtData(DataSource):
 
     def _parse_source_symbol_bars(self, response, asset, quote=None, length=None):
         # Parse the dataframe returned from CCXT.
-        response["return"] = response["close"].pct_change()
         bars = Bars(response, self.SOURCE, asset, quote=quote, raw=response)
         return bars
 
-    def get_last_price(self, asset, quote=None, exchange=None, **kwargs):
+    def get_last_price(self, asset, quote=None, exchange=None, **kwargs) -> Union[float, Decimal, None]:
         if quote is not None:
             symbol = f"{asset.symbol}/{quote.symbol}"
         else:
@@ -204,4 +220,4 @@ class CcxtData(DataSource):
         ticker = self.api.fetch_ticker(symbol)
         price = ticker["last"]
 
-        return price
+        return price
\ No newline at end of file
