@@ -1,11 +1,23 @@
 import datetime
 import math
+from decimal import Decimal
+from typing import Union
 
 import pandas as pd
-from lumibot.entities import Bars
+
+from lumibot.entities import Asset, Bars, Quote
 
 from .data_source import DataSource
 
+TYPE_MAP = dict(
+    stock="STK",
+    option="OPT",
+    future="FUT",
+    forex="CASH",
+    index="IND",
+    multileg="BAG",
+)
+
 
 class InteractiveBrokersData(DataSource):
     """Make Interactive Brokers connection and gets data.
@@ -79,6 +91,13 @@ class InteractiveBrokersData(DataSource):
         },
     ]
 
+    def __init__(self, config, max_workers=20, chunk_size=100, **kwargs):
+        super().__init__(**kwargs)
+        self.name = "interactivebrokers"
+        self.max_workers = min(max_workers, 200)
+        self.chunk_size = min(chunk_size, 100)
+        self.ib = None
+
     @staticmethod
     def _format_datetime(dt):
         return pd.Timestamp(dt).isoformat()
@@ -87,11 +106,6 @@ class InteractiveBrokersData(DataSource):
     def _format_ib_datetime(dt):
         return pd.Timestamp(dt).strftime("%Y%m%d %H:%M:%S")
 
-    def __init__(self, config, max_workers=20, chunk_size=100, **kwargs):
-        self.name = "interactivebrokers"
-        self.max_workers = min(max_workers, 200)
-        self.chunk_size = min(chunk_size, 100)
-
     def _parse_duration(self, length, timestep):
         # If the timestemp includes a number, then separate it from the unit.
         if timestep[0].isdigit():
@@ -123,9 +137,7 @@ class InteractiveBrokersData(DataSource):
         elif "day" in freq:
             return f"{str(length)} D"
         else:
-            raise ValueError(
-                f"Timestep must be `day` or `minute`, you entered: {timestep}"
-            )
+            raise ValueError(f"Timestep must be `day` or `minute`, you entered: {timestep}")
 
     def _pull_source_symbol_bars(
         self,
@@ -230,16 +242,16 @@ class InteractiveBrokersData(DataSource):
 
                 if "min" in parsed_timestep or "hour" in parsed_timestep:
                     df["date"] = (
-                        pd.to_datetime(df["date"], unit="s", origin="unix")
+                        pd.to_datetime(df["date"].astype(int), unit="s", origin="unix")
                         .dt.tz_localize("UTC")
                         .dt.tz_convert(self.DEFAULT_TIMEZONE)
                     )
-                    df = df.iloc[-length:, :]
+                    df = df.iloc[-int(length) :, :]
                 elif "day" in parsed_timestep:
                     df["date"] = pd.to_datetime(df["date"], format="%Y%m%d")
                     df["date"] = df["date"].dt.tz_localize(self.DEFAULT_TIMEZONE)
 
-                    df = df.iloc[-length:, :]
+                    df = df.iloc[-int(length) :, :]
 
                 response[asset] = df
         return response
@@ -251,7 +263,8 @@ class InteractiveBrokersData(DataSource):
             return bars
         df = response.copy()
         # df["date"] = pd.to_datetime(df["date"], unit='s')
-        df = df.set_index("date")
+        df = df.rename(columns={"date": "datetime"})  # Renaming to match other data sources like BackTest and Alpaca.
+        df = df.set_index("datetime")
         df["price_change"] = df["close"].pct_change()
         df["dividend"] = 0
         df["stock_splits"] = 0
@@ -290,7 +303,46 @@ class InteractiveBrokersData(DataSource):
         self.ib.cancel_realtime_bars(asset)
         return 0
 
-    def get_last_price(self, asset, timestep=None, quote=None, exchange=None, **kwargs):
+    def get_chains(self, asset: Asset, quote: Asset = None, exchange: str = None):
+        """
+        For InteractiveBrokers, this function is defined in the broker because the broker object has access
+        to additional API calls that are not available in the data source object because of the way IBClient and
+        IBWrapper are designed.
+        """
+        raise NotImplementedError(
+            "Lumibot InteractiveBrokersData does not support get_chains options data. However, this is defined in"
+            "the InteractiveBroker broker class as it has access to IBClient and IBWrapper functionality not available"
+            "here. If you need this feature, please call the broker object method directly"
+        )
+
+    def get_historical_prices(
+        self, asset, length, timestep="", timeshift=None, quote=None, exchange=None, include_after_hours=True
+    ):
+        """Get bars for a given asset"""
+        if isinstance(asset, str):
+            asset = Asset(symbol=asset)
+
+        if not timestep:
+            timestep = self.get_timestep()
+
+        response = self._pull_source_symbol_bars(
+            asset,
+            length,
+            timestep=timestep,
+            timeshift=timeshift,
+            quote=quote,
+            exchange=exchange,
+            include_after_hours=include_after_hours,
+        )
+        if isinstance(response, float):
+            return response
+        elif response is None:
+            return None
+
+        bars = self._parse_source_symbol_bars(response, asset, quote=quote, length=length)
+        return bars
+
+    def get_last_price(self, asset, timestep=None, quote=None, exchange=None, **kwargs) -> Union[float, Decimal, None]:
         if exchange is None:
             exchange = "SMART"
 
@@ -307,9 +359,10 @@ class InteractiveBrokersData(DataSource):
                     asset,
                     exchange=exchange,
                     should_use_last_close=should_use_last_close,
+                    only_price=False,
                 )
                 if result:
-                    response[asset] = result[0]
+                    response[asset] = result["price"]
                     break
                 get_data_attempt += 1
             except:
@@ -323,9 +376,59 @@ class InteractiveBrokersData(DataSource):
         return result
 
     def get_yesterday_dividend(self, asset, quote=None):
-        """ Unavailable """
+        """Unavailable"""
         return 0
 
     def get_yesterday_dividends(self, asset, quote=None):
-        """ Unavailable """
+        """Unavailable"""
         return None
+
+    def get_quote(self, asset, quote=None, exchange=None) -> Quote:
+        """
+        This function returns the quote of an asset. The quote includes the bid and ask price.
+
+        Parameters
+        ----------
+        asset: Asset
+            The asset to get the quote for
+        quote: Asset
+            The quote asset to get the quote for (currently not used for Interactive Brokers)
+        exchange: str
+            The exchange to get the quote from
+
+        Returns
+        -------
+        Quote
+           Quote object containing bid, ask, price and other information
+        """
+        from lumibot.entities import Quote
+
+        if exchange is None:
+            exchange = "SMART"
+
+        get_data_attempt = 0
+        max_attempts = 2
+        while get_data_attempt < max_attempts:
+            try:
+                result = self.ib.get_tick(asset, exchange=exchange, only_price=False)
+                if result:
+                    # If bid or ask are -1 then they are not available.
+                    bid = None if result["bid"] == -1 else result["bid"]
+                    ask = None if result["ask"] == -1 else result["ask"]
+
+                    # Create and return Quote object
+                    return Quote(
+                        asset=asset,
+                        price=result.get("price"),
+                        bid=bid,
+                        ask=ask,
+                        volume=result.get("volume"),
+                        timestamp=datetime.datetime.now(datetime.timezone.utc),
+                        raw_data=result
+                    )
+                get_data_attempt += 1
+            except:
+                get_data_attempt += 1
+
+        # Return empty Quote if we couldn't get data
+        return Quote(asset=asset)
